import Joi from 'joi';
import { Request, Response, NextFunction } from 'express';
import { UserType } from '@/types';

// User validation schemas
export const registerSchema = Joi.object({
  email: Joi.string().email().required().messages({
    'string.email': 'Please provide a valid email address',
    'any.required': 'Email is required'
  }),
  password: Joi.string().min(8).pattern(new RegExp('^(?=.*[a-z])(?=.*[A-Z])(?=.*\\d)')).required().messages({
    'string.min': 'Password must be at least 8 characters long',
    'string.pattern.base': 'Password must contain at least one uppercase letter, one lowercase letter, and one number',
    'any.required': 'Password is required'
  }),
  fullName: Joi.string().min(2).max(100).required().messages({
    'string.min': 'Full name must be at least 2 characters long',
    'string.max': 'Full name must not exceed 100 characters',
    'any.required': 'Full name is required'
  }),
  userType: Joi.string().valid(...Object.values(UserType)).required().messages({
    'any.only': 'User type must be either DEVELOPER or BUYER',
    'any.required': 'User type is required'
  })
});

export const loginSchema = Joi.object({
  email: Joi.string().email().required().messages({
    'string.email': 'Please provide a valid email address',
    'any.required': 'Email is required'
  }),
  password: Joi.string().required().messages({
    'any.required': 'Password is required'
  })
});

// Wallet validation schemas
export const walletLinkSchema = Joi.object({
  walletAddress: Joi.string().pattern(new RegExp('^0x[a-fA-F0-9]{40}$')).required().messages({
    'string.pattern.base': 'Please provide a valid Ethereum wallet address',
    'any.required': 'Wallet address is required'
  })
});

// Project validation schemas
export const createProjectSchema = Joi.object({
  title: Joi.string().min(5).max(200).required().messages({
    'string.min': 'Project title must be at least 5 characters long',
    'string.max': 'Project title must not exceed 200 characters',
    'any.required': 'Project title is required'
  }),
  description: Joi.string().min(20).max(2000).required().messages({
    'string.min': 'Project description must be at least 20 characters long',
    'string.max': 'Project description must not exceed 2000 characters',
    'any.required': 'Project description is required'
  }),
  location: Joi.string().min(2).max(100).required().messages({
    'string.min': 'Location must be at least 2 characters long',
    'string.max': 'Location must not exceed 100 characters',
    'any.required': 'Location is required'
  }),
  projectType: Joi.string().min(2).max(50).required().messages({
    'string.min': 'Project type must be at least 2 characters long',
    'string.max': 'Project type must not exceed 50 characters',
    'any.required': 'Project type is required'
  }),
  methodology: Joi.string().min(2).max(100).required().messages({
    'string.min': 'Methodology must be at least 2 characters long',
    'string.max': 'Methodology must not exceed 100 characters',
    'any.required': 'Methodology is required'
  }),
  totalCreditsAvailable: Joi.number().integer().min(1).max(1000000).required().messages({
    'number.base': 'Total credits available must be a number',
    'number.integer': 'Total credits available must be an integer',
    'number.min': 'Total credits available must be at least 1',
    'number.max': 'Total credits available must not exceed 1,000,000',
    'any.required': 'Total credits available is required'
  }),
  pricePerCredit: Joi.number().min(0.01).max(10000).required().messages({
    'number.base': 'Price per credit must be a number',
    'number.min': 'Price per credit must be at least $0.01',
    'number.max': 'Price per credit must not exceed $10,000',
    'any.required': 'Price per credit is required'
  }),
  startDate: Joi.date().iso().required().messages({
    'date.base': 'Start date must be a valid date',
    'date.format': 'Start date must be in ISO format',
    'any.required': 'Start date is required'
  }),
  endDate: Joi.date().iso().greater(Joi.ref('startDate')).required().messages({
    'date.base': 'End date must be a valid date',
    'date.format': 'End date must be in ISO format',
    'date.greater': 'End date must be after start date',
    'any.required': 'End date is required'
  })
});

export const updateProjectSchema = Joi.object({
  title: Joi.string().min(5).max(200).optional(),
  description: Joi.string().min(20).max(2000).optional(),
  location: Joi.string().min(2).max(100).optional(),
  projectType: Joi.string().min(2).max(50).optional(),
  methodology: Joi.string().min(2).max(100).optional(),
  totalCreditsAvailable: Joi.number().integer().min(1).max(1000000).optional(),
  pricePerCredit: Joi.number().min(0.01).max(10000).optional(),
  startDate: Joi.date().iso().optional(),
  endDate: Joi.date().iso().optional(),
  verificationDocumentUrl: Joi.string().uri().optional().messages({
    'string.uri': 'Verification document URL must be a valid URL'
  })
});

// Transaction validation schemas
export const purchaseSchema = Joi.object({
  projectId: Joi.number().integer().min(1).required().messages({
    'number.base': 'Project ID must be a number',
    'number.integer': 'Project ID must be an integer',
    'number.min': 'Project ID must be at least 1',
    'any.required': 'Project ID is required'
  }),
  quantity: Joi.number().integer().min(1).max(100000).required().messages({
    'number.base': 'Quantity must be a number',
    'number.integer': 'Quantity must be an integer',
    'number.min': 'Quantity must be at least 1',
    'number.max': 'Quantity must not exceed 100,000',
    'any.required': 'Quantity is required'
  })
});

// Pagination validation schema
export const paginationSchema = Joi.object({
  page: Joi.number().integer().min(1).default(1).optional(),
  pageSize: Joi.number().integer().min(1).max(100).default(10).optional()
});

// Validation middleware factory
export const validate = (schema: Joi.ObjectSchema) => {
  return (req: Request, res: Response, next: NextFunction) => {
    const { error, value } = schema.validate(req.body, {
      abortEarly: false,
      stripUnknown: true
    });

    if (error) {
      const errors = error.details.map(detail => ({
        field: detail.path.join('.'),
        message: detail.message
      }));

      return res.status(400).json({
        success: false,
        error: 'Validation failed',
        details: errors
      });
    }

    req.body = value;
    next();
  };
};

// Query validation middleware
export const validateQuery = (schema: Joi.ObjectSchema) => {
  return (req: Request, res: Response, next: NextFunction) => {
    const { error, value } = schema.validate(req.query, {
      abortEarly: false,
      stripUnknown: true
    });

    if (error) {
      const errors = error.details.map(detail => ({
        field: detail.path.join('.'),
        message: detail.message
      }));

      return res.status(400).json({
        success: false,
        error: 'Query validation failed',
        details: errors
      });
    }

    req.query = value;
    next();
  };
};