import { Router, Request, Response } from 'express';
import { prisma } from '@/utils/database';
import { AuthUtils } from '@/utils/auth';
import { validate, registerSchema, loginSchema } from '@/middleware/validation';
import { CreateUserRequest, LoginRequest, UserResponse, ApiResponse } from '@/types';

const router = Router();

// Register endpoint
router.post('/register', validate(registerSchema), async (req: Request, res: Response) => {
  try {
    const { email, password, fullName, userType }: CreateUserRequest = req.body;

    // Check if user already exists
    const existingUser = await prisma.user.findUnique({
      where: { email }
    });

    if (existingUser) {
      return res.status(400).json({
        success: false,
        error: 'Email already registered'
      } as ApiResponse);
    }

    // Hash password
    const hashedPassword = await AuthUtils.hashPassword(password);

    // Create user
    const user = await prisma.user.create({
      data: {
        email,
        password: hashedPassword,
        fullName,
        userType
      },
      select: {
        id: true,
        email: true,
        fullName: true,
        userType: true,
        isActive: true,
        createdAt: true,
        walletAddress: true
      }
    });

    return res.status(201).json({
      success: true,
      data: user,
      message: 'User registered successfully'
    } as ApiResponse<UserResponse>);

  } catch (error) {
    console.error('Registration error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during registration'
    } as ApiResponse);
  }
});

// Login endpoint
router.post('/login', validate(loginSchema), async (req: Request, res: Response) => {
  try {
    const { email, password }: LoginRequest = req.body;

    // Find user
    const user = await prisma.user.findUnique({
      where: { email }
    });

    if (!user) {
      return res.status(401).json({
        success: false,
        error: 'Invalid credentials'
      } as ApiResponse);
    }

    // Verify password
    const isPasswordValid = await AuthUtils.verifyPassword(password, user.password);
    if (!isPasswordValid) {
      return res.status(401).json({
        success: false,
        error: 'Invalid credentials'
      } as ApiResponse);
    }

    // Check if user is active
    if (!user.isActive) {
      return res.status(401).json({
        success: false,
        error: 'Account is deactivated'
      } as ApiResponse);
    }

    // Generate JWT token
    const token = AuthUtils.generateAccessToken({
      userId: user.id,
      email: user.email,
      userType: user.userType
    });

    return res.status(200).json({
      success: true,
      data: {
        accessToken: token,
        tokenType: 'Bearer',
        user: {
          id: user.id,
          email: user.email,
          fullName: user.fullName,
          userType: user.userType,
          isActive: user.isActive,
          createdAt: user.createdAt,
          walletAddress: user.walletAddress
        }
      },
      message: 'Login successful'
    } as ApiResponse);

  } catch (error) {
    console.error('Login error:', error);
    return res.status(500).json({
      success: false,
      error: 'Internal server error during login'
    } as ApiResponse);
  }
});

export default router;