import Web3 from 'web3';
import { ethers } from 'ethers';
import { BlockchainTransaction, WalletInfo } from '@/types';

export class BlockchainService {
  private web3: Web3;
  private provider: ethers.JsonRpcProvider;
  private contractAbi: any[];

  constructor() {
    const rpcUrl = process.env.BLOCKCHAIN_RPC_URL || 'http://localhost:8545';
    this.web3 = new Web3(rpcUrl);
    this.provider = new ethers.JsonRpcProvider(rpcUrl);
    this.contractAbi = this.getCarbonTokenAbi();
  }

  private getCarbonTokenAbi(): any[] {
    // Simplified ABI for a carbon credit token contract
    return [
      {
        inputs: [
          { name: 'to', type: 'address' },
          { name: 'tokenId', type: 'uint256' },
          { name: 'credits', type: 'uint256' }
        ],
        name: 'mintCarbonCredit',
        outputs: [{ name: '', type: 'bool' }],
        type: 'function'
      },
      {
        inputs: [
          { name: 'from', type: 'address' },
          { name: 'to', type: 'address' },
          { name: 'tokenId', type: 'uint256' }
        ],
        name: 'transferFrom',
        outputs: [{ name: '', type: 'bool' }],
        type: 'function'
      },
      {
        inputs: [{ name: 'tokenId', type: 'uint256' }],
        name: 'ownerOf',
        outputs: [{ name: '', type: 'address' }],
        type: 'function'
      }
    ];
  }

  public validateWalletAddress(address: string): boolean {
    try {
      return this.web3.utils.isAddress(address);
    } catch (error) {
      console.error('Error validating wallet address:', error);
      return false;
    }
  }

  public generateWallet(): WalletInfo {
    try {
      const wallet = ethers.Wallet.createRandom();
      return {
        address: wallet.address,
        privateKey: wallet.privateKey,
        publicKey: wallet.publicKey
      };
    } catch (error) {
      console.error('Error generating wallet:', error);
      throw new Error('Failed to generate wallet');
    }
  }

  public async getWalletBalance(address: string): Promise<number | null> {
    try {
      if (!this.validateWalletAddress(address)) {
        return null;
      }

      const balanceWei = await this.web3.eth.getBalance(address);
      const balanceEth = this.web3.utils.fromWei(balanceWei, 'ether');
      return parseFloat(balanceEth);
    } catch (error) {
      console.error(`Error getting balance for ${address}:`, error);
      return null;
    }
  }

  public async createCarbonTokenTransaction(
    contractAddress: string,
    fromAddress: string,
    toAddress: string,
    tokenId: number
  ): Promise<BlockchainTransaction | null> {
    try {
      if (!this.validateWalletAddress(contractAddress) ||
          !this.validateWalletAddress(fromAddress) ||
          !this.validateWalletAddress(toAddress)) {
        return null;
      }

      const contract = new this.web3.eth.Contract(this.contractAbi, contractAddress);
      const data = contract.methods.transferFrom(fromAddress, toAddress, tokenId).encodeABI();
      
      const gasPrice = await this.web3.eth.getGasPrice();
      const gasLimit = await contract.methods.transferFrom(fromAddress, toAddress, tokenId)
        .estimateGas({ from: fromAddress });

      return {
        to: contractAddress,
        value: '0',
        data,
        gasLimit: gasLimit.toString(),
        gasPrice: gasPrice.toString()
      };
    } catch (error) {
      console.error('Error creating transaction:', error);
      return null;
    }
  }

  public async signAndSendTransaction(
    transaction: BlockchainTransaction,
    privateKey: string
  ): Promise<string | null> {
    try {
      const wallet = new ethers.Wallet(privateKey, this.provider);
      const tx = await wallet.sendTransaction({
        to: transaction.to,
        value: transaction.value,
        data: transaction.data,
        gasLimit: transaction.gasLimit,
        gasPrice: transaction.gasPrice
      });

      return tx.hash;
    } catch (error) {
      console.error('Error signing/sending transaction:', error);
      return null;
    }
  }

  public async getTransactionReceipt(txHash: string): Promise<any | null> {
    try {
      const receipt = await this.web3.eth.getTransactionReceipt(txHash);
      return {
        transactionHash: receipt.transactionHash,
        blockNumber: receipt.blockNumber,
        gasUsed: receipt.gasUsed,
        status: receipt.status
      };
    } catch (error) {
      console.error('Error getting transaction receipt:', error);
      return null;
    }
  }

  public async verifyTokenOwnership(
    contractAddress: string,
    tokenId: number,
    ownerAddress: string
  ): Promise<boolean> {
    try {
      if (!this.validateWalletAddress(contractAddress) ||
          !this.validateWalletAddress(ownerAddress)) {
        return false;
      }

      const contract = new this.web3.eth.Contract(this.contractAbi, contractAddress);
      const actualOwner = await contract.methods.ownerOf(tokenId).call();
      
      return actualOwner.toLowerCase() === ownerAddress.toLowerCase();
    } catch (error) {
      console.error('Error verifying ownership:', error);
      return false;
    }
  }

  public async isNetworkConnected(): Promise<boolean> {
    try {
      await this.web3.eth.getBlockNumber();
      return true;
    } catch (error) {
      console.error('Blockchain network connection failed:', error);
      return false;
    }
  }
}

export const blockchainService = new BlockchainService();